# Generated by Django 2.0.2 on 2018-02-23 01:17

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='UsernameBlacklist',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('username', models.CharField(help_text='Username to black list', max_length=250, unique=True, verbose_name='username')),
                ('is_blocked', models.BooleanField(default=True, help_text="Set this to False if you need to whitelist this username but don't want to delete it. This also ensures that a whitelisted username will stay whitelisted when re-populating using the `populate` manager method.", verbose_name='is blocked')),
            ],
        ),
    ]



from django.contrib import admin
from username_tools.models import UsernameBlacklist


@admin.register(UsernameBlacklist)
class UsernameBlacklistAdmin(admin.ModelAdmin):
    list_display = ('username', 'is_blocked')
    list_filter = ('is_blocked',)



from django.apps import apps
from username_tools.blacklist import BLACKLIST

# to cache the model class after the first import
UsernameBlacklist = None


def blacklist_model():
    global UsernameBlacklist

    if UsernameBlacklist is None:
        from username_tools.models import UsernameBlacklist
    return UsernameBlacklist


def is_blacklisted(value):
    if apps.is_installed("username_tools"):
        return blacklist_model().objects.is_blacklisted(value)
    return value.lower() in BLACKLIST



import unicodedata

from django import forms
from django.db import models
from django.utils.translation import gettext_lazy as _
from django.core.exceptions import ValidationError
from django.contrib.auth.validators import UnicodeUsernameValidator
from username_tools.utils import is_blacklisted
from username_tools.settings import (
    BLACKLIST_VALIDATION_ERROR_MESSAGE,
    USERNAME_MIN_LENGTH
)


class UsernameFormField(forms.CharField):
    """
    A `forms.CharField` subclass that comes with validation for blacklisted usernames.
    """

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('min_length', USERNAME_MIN_LENGTH)
        super().__init__(*args, **kwargs)

    def to_python(self, value):
        value = unicodedata.normalize('NFKC', super().to_python(value))
        if is_blacklisted(value):
            raise ValidationError(BLACKLIST_VALIDATION_ERROR_MESSAGE)
        return value


class UsernameModelField(models.CharField):
    """
    A `models.CharField` subclass that comes with validation for blacklisted usernames.
    """
    default_validators = [UnicodeUsernameValidator()]
    default_error_messages = {
        'unique': _("A user with that username already exists."),
    }

    def __init__(self, *args, **kwargs):
        # defaults are copied from django.contrib.auth's username field
        defaults = {
            'verbose_name': _('username'),
            'max_length': 150,
            'unique': True,
            'help_text': _('Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.'),
        }
        defaults.update(**kwargs)
        super().__init__(*args, **defaults)

    def formfield(self, **kwargs):
        defaults = {'form_class': UsernameFormField}
        defaults.update(kwargs)
        return super().formfield(**defaults)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
import sys
from setuptools import setup


def get_version(*file_paths):
    """Retrieves the version from username_tools/__init__.py"""
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


version = get_version("username_tools", "__init__.py")


if sys.argv[-1] == 'publish':
    try:
        import twine
        print("Twine version: ", twine.__version__)
    except ImportError:
        print('Twine library missing. Please run "pip install twine"')
        sys.exit()
    os.system('python setup.py sdist')
    os.system('twine upload dist/*')
    sys.exit()

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

setup(
    name='django-username-tools',
    version=version,
    description="""Username validation fields and utilities for Django""",
    long_description=readme + '\n\n' + history,
    author='keshaB Paudel',
    author_email='self@keshab.net',
    url='https://github.com/poudel/django-username-tools',
    packages=[
        'username_tools',
    ],
    include_package_data=True,
    install_requires=["Django>=2.0.2",],
    license="MIT",
    zip_safe=False,
    keywords='django-username-tools',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Framework :: Django :: 2.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)



from django.conf import settings
from django.utils.translation import gettext_lazy as _


BLACKLIST_VALIDATION_ERROR_MESSAGE = getattr(
    settings,
    "BLACKLIST_VALIDATION_ERROR_MESSAGE",
    _("Username not available.")
)


EMAIL_IS_USERNAME = getattr(
    settings,
    "EMAIL_IS_USERNAME",
    False
)


USERNAME_MIN_LENGTH = getattr(
    settings,
    "USERNAME_MIN_LENGTH",
    6
)



from django.db import models
from django.utils.translation import gettext_lazy as _
from django.contrib.auth import get_user_model
from username_tools.blacklist import BLACKLIST


UserModel = get_user_model()


class UsernameBlacklistManager(models.Manager):

    def is_blacklisted(self, username):
        lookup = "{}__iexact".format(UserModel.USERNAME_FIELD)
        fil = {lookup: username, 'is_blocked': True}
        return self.filter(**fil).exists()

    def blacklisted(self, *args, **kwargs):
        return self.filter(*args, is_blocked=True, **kwargs)

    def populate(self):
        """
        Populate database from file.

        Set `reset` to True if you want to update those
        that you have modified.
        """
        blacklist = set(BLACKLIST)

        saved = self.values_list('username', flat=True)
        if saved:
            blacklist = blacklist.difference(saved)

        objs = [self.model(username=u) for u in blacklist]
        self.bulk_create(objs)
        return len(objs)


class UsernameBlacklist(models.Model):
    username = models.CharField(
        _("username"),
        help_text=_("Username to black list"),
        max_length=250,
        unique=True
    )
    is_blocked = models.BooleanField(
        _("is blocked"),
        default=True,
        help_text=_(
            "Set this to False if you need to whitelist this username but don't "
            "want to delete it. This also ensures that a whitelisted username "
            "will stay whitelisted when re-populating using the `populate` "
            "manager method."
        )
    )

    objects = UsernameBlacklistManager()

    def __str__(self):
        return self.username
